package de.intevation.gnv.transition;

import java.util.Iterator;

import org.apache.log4j.Logger;
import org.w3c.dom.Node;

import de.intevation.artifactdatabase.Config;
import de.intevation.gnv.state.InputData;
import de.intevation.gnv.state.State;
import de.intevation.gnv.state.exception.StateException;

/**
 * Class which supports the possibility to validate if a
 * Transition could be done using simple Valuecompare Operations.
 * At this Time equal and notequal-operations are supported.
 * The Comparation is casesensitive.
 * @author <a href="mailto:tim.englich@intevation.de">Tim Englich</a>
 *
 */
public class ValueCompareTransition extends TransitionBase {

    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(ValueCompareTransition.class);
    /**
     * The UID of this Class.
     */
    private static final long serialVersionUID = -7846722158776823205L;

    /**
     * The Name of the Datafield the Value  that should be compared 
     * should be fetched from.
     */
    private String dataName = null;
    /**
     * The Value that should be set to the State.
     */
    private String dataValue = null;
    /**
     * The Operation that should be used (equal notequal)
     */
    private String operator = null;


    /**
     * Constructor
     */
    public ValueCompareTransition() {
        super();
    }

    /**
     * validates if a Transition is valid and could be used using simple
     * Comparevalue-Operations.
     * @see de.intevation.gnv.transition.Transition#isValid(de.intevation.gnv.state.State)
     */
    public boolean isValid(State state) {

        try {
            Iterator<InputData> it = state.getInputData().iterator();
            while (it.hasNext()){
                InputData inputData = it.next();
                if (inputData.getName().equals(this.dataName)){
                    boolean returnValue = false;
                    if (operator.equals("equal")){
                        returnValue = this.dataValue.equals(inputData.getValue());
                    }else if (operator.equals("notequal")){
                        returnValue = !this.dataValue.equals(inputData.getValue());
                    }
                    return returnValue;
                }
            }
            if (operator.equals("notequal")){
                // data is not given. So the constraint not Equals is fullfilled.
                return true;
            }
        } catch (StateException e) {
            log.error(e,e);
            return false;
        }
        return false;
    }

    
    @Override
    public void setup(Node configuration) {
        super.setup(configuration);
        this.dataName = Config.getStringXPath(configuration,"condition/@inputvalue");
        this.dataValue = Config.getStringXPath(configuration,"condition/@value");
        this.operator = Config.getStringXPath(configuration,"condition/@operator");
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :